/**
 * Created by Ricardo Morais on 02/03/2017.
 */
let Sequelize = require('sequelize');
let object = {

    init: (dialect, host, user, password, database) => {
        let entityCore = require('entity-core').init(dialect, host, user, password, database);
        object.adapt(entityCore);
    },
    adapt: (entityCore) => {

        let tablePrefix = 'EntityInstanceCore';

        let meta = {
            sequelize: entityCore.sequelize,
            name: 'entity-instance-core',
            dependencies: {
                entityCore: entityCore,
            },

            query: {
                action: {
                    createEntityInstance: (entityID) => {
                        meta.model.EntityInstance.create({
                            entityID: entityID
                        });
                    }
                }
            },
            model: {
                EntityInstance: sequelize.define(tablePrefix + 'EntityInstance', {
                    entityID: { type: Sequelize.INTEGER, allowNull: false},
                }, {
                    freezeTableName: true,
                    underscoredAll: true
                }),
                EntityInstancePrimitiveValue: sequelize.define(tablePrefix + 'EntityInstancePrimitiveValue', {
                    entityInstanceID: { type: Sequelize.INTEGER, allowNull: false},
                    primitiveAttributeEntityID: { type: Sequelize.INTEGER, allowNull: false},
                    value: { type: Sequelize.TEXT, allowNull: false, unique: true},
                }, {
                    freezeTableName: true,
                    underscoredAll: true
                }),
                EntityInstanceReferenceValue: sequelize.define(tablePrefix + 'EntityInstanceReferenceValue', {
                    entityInstanceID: { type: Sequelize.INTEGER, allowNull: false},
                    referenceAttributeEntityID: { type: Sequelize.INTEGER, allowNull: false},
                    referenceEntityInstanceID: { type: Sequelize.INTEGER, allowNull: false},
                }, {
                    freezeTableName: true,
                    underscoredAll: true
                }),
            },
            primitiveAttributes: {},
        };

        meta.model.EntityInstancePrimitiveValue.belongsTo(entityCore.model.PrimitiveAttributeEntity, {
            foreignKey: 'primitiveAttributeEntityID',
            constraints: false,
        });
        meta.model.EntityInstancePrimitiveValue.belongsTo(meta.model.EntityInstance, {
            foreignKey: 'entityInstanceID',
            constraints: false,
        });
        meta.model.EntityInstanceReferenceValue.belongsTo(entityCore.model.ReferenceAttributeEntity, {
            foreignKey: 'referenceAttributeEntityID',
            constraints: false,
        });
        meta.model.EntityInstanceReferenceValue.belongsTo(meta.model.EntityInstance, {
            foreignKey: 'entityInstanceID',
            constraints: false,
        });
        meta.model.EntityInstanceReferenceValue.belongsTo(meta.model.EntityInstance, {
            foreignKey: 'referenceEntityInstanceID',
            constraints: false,
        });

        //Create the primitive attribute types
        let promise = new Promise((resolve, reject)=>{
            sequelize.sync({force: true}).then(()=> {
                meta.query.action.createPrimitiveAttributeType('number').then((number) => {
                    meta.primitiveAttributes.NUMBER = number;
                    return meta.query.action.createPrimitiveAttributeType('string');
                }).then((string) => {
                    meta.primitiveAttributes.STRING = string;
                    return meta.query.action.createPrimitiveAttributeType('date');
                }).then((date) => {
                    meta.primitiveAttributes.DATE = date;
                    return meta.query.action.createPrimitiveAttributeType('json');
                }).then((json) => {
                    meta.primitiveAttributes.JSON = json;
                    resolve(meta);
                });
            });
        });

        return promise;
    }
};

module.exports = object;
